package com.persagy.apm.report.dependencies.centermiddleware.service.impl;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.dependencies.centermiddleware.client.CenterMiddlewareWebClient;
import com.persagy.apm.report.dependencies.centermiddleware.model.dto.Criteria;
import com.persagy.apm.report.dependencies.centermiddleware.model.dto.QueryEnergyDataDTO;
import com.persagy.apm.report.dependencies.centermiddleware.model.dto.QueryObjectInfoDTO;
import com.persagy.apm.report.dependencies.centermiddleware.model.vo.EnergyDataInfoVO;
import com.persagy.apm.report.dependencies.centermiddleware.model.vo.ObjectInfoVO;
import com.persagy.apm.report.dependencies.centermiddleware.service.ICenterMiddlewareWebService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

@Service
@Slf4j
public class ICenterMiddlewareWebServiceImp implements ICenterMiddlewareWebService {

    @Autowired
    private CenterMiddlewareWebClient centerMiddlewareWebClient;

    @Override
    public Map<String, TreeMap<Date, Double>> getItemEnergyDataMap(QueryEnergyDataDTO queryEnergyDataDTO) {
        if (null == queryEnergyDataDTO) {
            return new HashMap<>();
        }
        Map<String, TreeMap<Date, Double>> resultMap = new HashMap<>();
        try {
            PoemsFeignResponse<EnergyDataInfoVO> response = centerMiddlewareWebClient.getItemEnergyData(queryEnergyDataDTO);
            List<EnergyDataInfoVO> energyDataInfoVOList = response.getContent();
            if (CollectionUtils.isEmpty(energyDataInfoVOList)) {
                return new HashMap<>();
            }
            for (EnergyDataInfoVO energyDataInfoVO : energyDataInfoVOList) {
                TreeMap<Date, Double> dateDataMap = resultMap.get(energyDataInfoVO.getItemId());
                if (null == dateDataMap) {
                    dateDataMap = new TreeMap<>();
                }
                dateDataMap.put(DateUtils.str2Date(energyDataInfoVO.getTime(), DateUtils.SDF_SECOND), energyDataInfoVO.getData());
                resultMap.put(energyDataInfoVO.getItemId(), dateDataMap);
            }
        } catch (Exception e) {
            throw new RuntimeException("数据平台获取能耗数据失败", e);
        }
        return resultMap;
    }


    private List<String> getObjectIdList(String pjId, String groupCode, List<String> classCodeList) {
        log.info("查询对象信息列表开始,项目：" + pjId + "groupCode:" + groupCode + "classCodeList:" + classCodeList);
        if (StringUtils.isEmpty(pjId) || StringUtils.isEmpty(groupCode) || CollectionUtils.isEmpty(classCodeList)) {
            log.info("查询中台对象信息时参数为空，故不进行查询");
            return new ArrayList<>();
        }
        List<String> resultList = new ArrayList<>();
        try {
            QueryObjectInfoDTO queryObjectInfoDTO = new QueryObjectInfoDTO();
            queryObjectInfoDTO.setProjectId(pjId);
            queryObjectInfoDTO.setGroupCode(groupCode);
            Criteria criteria = new Criteria();
            criteria.setClassCode(classCodeList);
            queryObjectInfoDTO.setCriteria(criteria);
            PoemsFeignResponse<ObjectInfoVO> response = centerMiddlewareWebClient.getObjectInfo(queryObjectInfoDTO);
            List<ObjectInfoVO> objectInfoVOList = response.getContent();
            log.info("中台返回对象结果：" + objectInfoVOList);
            if (!CollectionUtils.isEmpty(objectInfoVOList)) {
                for (ObjectInfoVO vo : objectInfoVOList) {
                    resultList.add(vo.getId());
                }
            }
        } catch (Exception e) {
            throw new RuntimeException("查询中台对象信息时异常", e);
        }
        log.info("查询对象信息列表结束");
        return resultList;
    }
}
