package com.persagy.apm.report.dependencies.dmpalarm.client;

import com.persagy.apm.common.response.DmpPageResponse;
import com.persagy.apm.report.common.config.FeignConfig;
import com.persagy.apm.report.dependencies.dmpalarm.client.fallback.DmpAlarmClientFallbackFactory;
import com.persagy.apm.report.dependencies.dmpalarm.model.AlarmRecord;
import com.persagy.apm.report.dependencies.dmpalarm.model.dto.AlarmUrlParam;
import com.persagy.apm.report.dependencies.dmpalarm.model.dto.QueryAlarmRecordDTO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * 报警中台feign客户端
 *
 * @author lixing
 * @version V1.0 2021/8/4 2:41 下午
 **/
@FeignClient(name = "dmp-alarm", fallbackFactory = DmpAlarmClientFallbackFactory.class,
        configuration = FeignConfig.class)
public interface DmpAlarmClient {
    /**
     * 查询报警记录
     *
     * @param alarmUrlParam       报警请求 url 通用参数
     * @param queryAlarmRecordDTO 查询条件
     * @return 报警记录
     * @author lixing
     * @version V1.0 2021/8/4 3:04 下午
     */
    @PostMapping("/alarm/record/query")
    DmpPageResponse<AlarmRecord> queryAlarmRecord(
            @SpringQueryMap AlarmUrlParam alarmUrlParam,
            @RequestBody QueryAlarmRecordDTO queryAlarmRecordDTO);
}
