package com.persagy.apm.report.dependencies.environment.client;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.report.common.constants.FeignClientNameConstants;
import com.persagy.apm.report.common.config.FeignConfig;
import com.persagy.apm.report.dependencies.environment.client.fallback.EnvironmentWebClientFallbackFactory;
import com.persagy.apm.report.dependencies.environment.model.vo.EnvAlarmItemVO;
import com.persagy.apm.report.dependencies.environment.model.vo.EnvComplianceRateVO;
import feign.Headers;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * 环境健康feign客户端
 *
 * @author wangchenghong
 */
@FeignClient(name = FeignClientNameConstants.ENVIRONMENT, fallbackFactory = EnvironmentWebClientFallbackFactory.class,
        configuration = FeignConfig.class)
public interface EnvironmentWebClient {
    /**
     * 查询环境指标达标率
     *
     * @param queryEnvDTO 查询条件
     * @return 平均达标率
     * @author wch
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/GetEnvTypeRate")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<EnvComplianceRateVO> queryEnvComplianceRate(@RequestParam("jsonString") String queryEnvDTO);

    /**
     * 查询环境指标对应的报警条目
     *
     * @param queryAlarmItemsDTO 查询条件
     * @return 环境指标对应的报警条目
     * @author lixing
     * @version V1.0 2021/8/4 2:23 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/QueryAlarmTypeList")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<List<EnvAlarmItemVO>> queryAlarmItems(@RequestParam("jsonString") String queryAlarmItemsDTO);
}
