package com.persagy.apm.report.dependencies.environment.service.impl;

import com.alibaba.fastjson.JSON;
import com.google.gson.Gson;
import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.report.dependencies.environment.client.EnvironmentWebClient;
import com.persagy.apm.report.dependencies.environment.model.dto.QueryEnvAlarmItemsDTO;
import com.persagy.apm.report.dependencies.environment.model.dto.QueryEnvComplianceRateDTO;
import com.persagy.apm.report.dependencies.environment.model.vo.EnvAlarmItemVO;
import com.persagy.apm.report.dependencies.environment.model.vo.EnvComplianceRateVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 环境健康feign客户端包装类
 *
 * @author wangchenghong
 */
@Service
public class EnvironmentWebClientWrapper {
    @Autowired
    private EnvironmentWebClient environmentWebClient;

    /**
     * 查询环境指标达标率
     *
     * @param queryEnvComplianceRateDTO 查询条件
     * @return 环境指标达标率
     * @author lixing
     * @version V1.0 2021/8/4 10:47 上午
     */
    public PoemsFeignResponse<EnvComplianceRateVO> queryEnvComplianceRate(
            QueryEnvComplianceRateDTO queryEnvComplianceRateDTO) {
        return environmentWebClient.queryEnvComplianceRate(JSON.toJSONString(queryEnvComplianceRateDTO));
    }

    /**
     * 查询环境指标对应的报警条目
     *
     * @param queryEnvAlarmItemsDTO 查询条件
     * @return 环境指标对应的报警条目
     * @author lixing
     * @version V1.0 2021/8/4 10:47 上午
     */
    public PoemsFeignResponse<List<EnvAlarmItemVO>> queryEnvAlarmItems(
            QueryEnvAlarmItemsDTO queryEnvAlarmItemsDTO) {
        return environmentWebClient.queryAlarmItems(JSON.toJSONString(queryEnvAlarmItemsDTO));
    }
}
