package com.persagy.apm.report.dependencies.environment.service.impl;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.dependencies.environment.model.dto.QueryEnvAlarmItemsDTO;
import com.persagy.apm.report.dependencies.environment.model.dto.QueryEnvComplianceRateDTO;
import com.persagy.apm.report.dependencies.environment.model.vo.EnvAlarmItemVO;
import com.persagy.apm.report.dependencies.environment.model.vo.EnvComplianceRateVO;
import com.persagy.apm.report.dependencies.environment.service.IEnvironmentWebService;
import com.persagy.apm.report.indicator.model.entity.EnvDefine;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 环境指标service类
 *
 * @author lixing
 * @version V1.0 2021/8/4 10:57 上午
 */
@Service
@Slf4j
public class EnvironmentWebServiceImpl implements IEnvironmentWebService {
    @Autowired
    private EnvironmentWebClientWrapper environmentWebClientWrapper;


    @Override
    public Double getEnvComplianceRate(String projectId, Date date, EnvDefine envDefine) {
        Double avgRate = null;
        QueryEnvComplianceRateDTO paramDTO = new QueryEnvComplianceRateDTO();
        paramDTO.setProjectId(projectId);
        paramDTO.setEnvTypeId(envDefine.getType());
        paramDTO.setEnvParentTypeId(envDefine.getParentType());
        paramDTO.setStartTime(DateUtils.date2Str(date, DateUtils.SDF_SECOND));
        paramDTO.setEndTime(DateUtils.date2Str(DateUtils.getMonthOff(date, 1), DateUtils.SDF_SECOND));
        PoemsFeignResponse<EnvComplianceRateVO> response = environmentWebClientWrapper.queryEnvComplianceRate(paramDTO);
        List<EnvComplianceRateVO> envComplianceRateVOList = response.getContent();
        if (!CollectionUtils.isEmpty(envComplianceRateVOList)) {
            EnvComplianceRateVO envComplianceRateVO = envComplianceRateVOList.get(0);
            if (null != envComplianceRateVO) {
                return envComplianceRateVO.getAvgNum();
            }
        }
        return avgRate;
    }

    @Override
    public List<String> queryEnvAlarmItems(EnvDefine envDefine) {
        QueryEnvAlarmItemsDTO queryEnvAlarmItemsDTO = new QueryEnvAlarmItemsDTO();
        queryEnvAlarmItemsDTO.setEnvTypeId(envDefine.getType());
        PoemsFeignResponse<List<EnvAlarmItemVO>> response = environmentWebClientWrapper.
                queryEnvAlarmItems(queryEnvAlarmItemsDTO);
        List<List<EnvAlarmItemVO>> content = response.getContent();
        if (CollectionUtils.isEmpty(content)) {
            return null;
        }
        List<EnvAlarmItemVO> alarmItems = content.get(0);
        return alarmItems.stream().map(EnvAlarmItemVO::getId).collect(Collectors.toList());
    }
}
