package com.persagy.apm.report.dependencies.manualfilling.service.impl;

import com.google.common.collect.Maps;
import com.google.gson.Gson;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.userconfig.function.enums.FunctionDataSourceEnum;
import com.persagy.apm.report.userconfig.function.model.Function;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import com.persagy.apm.report.dependencies.manualfilling.model.dto.QueryEnergyItemDataDTO;
import com.persagy.apm.report.dependencies.manualfilling.model.dto.SaveEnergyItemDataDTO;
import com.persagy.apm.report.dependencies.manualfilling.model.vo.EnergyItemDataVo;
import com.persagy.apm.report.dependencies.manualfilling.model.vo.EnergyItemInfoVo;
import com.persagy.apm.report.dependencies.manualfilling.service.IEnergyItemDataService;
import com.persagy.apm.report.dependencies.saasweb.model.vo.ReportProjectVO;
import com.persagy.apm.report.dependencies.saasweb.service.ISaasWebService;
import com.persagy.apm.report.indicator.manualfilling.model.ManualFillingData;
import com.persagy.apm.report.indicator.manualfilling.model.dto.QueryManualFillingDataDTO;
import com.persagy.apm.report.indicator.manualfilling.service.IManualFillingDataService;
import com.persagy.apm.report.indicator.model.entity.BudgetDefine;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class IEnergyItemDataServiceImpl implements IEnergyItemDataService {
    @Autowired
    private IFunctionService functionService;
    @Autowired
    private ISaasWebService iSaasWebService;
    @Autowired
    private IManualFillingDataService manualFillingDataService;

    @Override
    public List<EnergyItemInfoVo> queryEnergyItemDataList(QueryEnergyItemDataDTO queryEnergyItemDataDTO) {
        try {
            // 初始化用户信息，本接口非前端调用，因此使用默认的用户信息
            PoemsContext.setContext(DataConstants.USER_ID, DataConstants.LOGIN_DEVICE, DataConstants.PD);
            // 查询项目信息
            ReportProjectVO projectVO = iSaasWebService.getReportProjectInfo(queryEnergyItemDataDTO.getProjectId());
            if (null == projectVO || StringUtils.isEmpty(projectVO.getFunctionType())) {
                return new ArrayList<>();
            }
            // 获取需要查询值的分项列表
            Map<String, Object> defineMap = Maps.newConcurrentMap();
            defineMap.put("energyType", queryEnergyItemDataDTO.getEnergyType());
            List<Function> functionList = functionService.queryByDefine(
                    FunctionDataSourceEnum.manualFilling.name(), defineMap);
            if (CollectionUtils.isEmpty(functionList)) {
                return new ArrayList<>();
            }
            List<String> functionIdList = functionList.stream().map(Function::getId).collect(Collectors.toList());

            // 查询分项的值
            QueryManualFillingDataDTO queryManualFillingDataDTO = new QueryManualFillingDataDTO();
            queryManualFillingDataDTO.setFunctionIds(functionIdList);
            queryManualFillingDataDTO.setType(queryEnergyItemDataDTO.getFunctionType());
            queryManualFillingDataDTO.setProjectId(queryEnergyItemDataDTO.getProjectId());
            queryManualFillingDataDTO.setStartTime(
                    DateUtils.str2Date(queryEnergyItemDataDTO.getStartDate(), DateUtils.SDF_SECOND));
            queryManualFillingDataDTO.setEndTime(
                    DateUtils.str2Date(queryEnergyItemDataDTO.getEndDate(), DateUtils.SDF_SECOND));
            List<ManualFillingData> manualFillingDataList = manualFillingDataService.
                    queryManualFillingDataList(queryManualFillingDataDTO);

            // {信息点id:{数据时间：数据}}
            Map<String, List<EnergyItemDataVo>> funcIdDateAndDataMap = new HashMap<>();
            if (CollectionUtils.isNotEmpty(manualFillingDataList)) {
                for (ManualFillingData manualFillingData : manualFillingDataList) {
                    List<EnergyItemDataVo> energyItemDataVoList = funcIdDateAndDataMap.get(manualFillingData.getFunctionId());
                    if (null == energyItemDataVoList) {
                        energyItemDataVoList = new ArrayList<>();
                    }
                    EnergyItemDataVo energyItemDataVo = new EnergyItemDataVo();
                    energyItemDataVo.setTime(DateUtils.date2Str(manualFillingData.getDataTime(), DateUtils.SDF_SECOND));
                    energyItemDataVo.setValue(manualFillingData.getValue());
                    energyItemDataVoList.add(energyItemDataVo);
                    funcIdDateAndDataMap.put(manualFillingData.getFunctionId(), energyItemDataVoList);
                }
            }

            // 将functionId转换为能耗模型信息
            List<EnergyItemInfoVo> resultList = new ArrayList<>();
            Gson gson = new Gson();
            for (Function function : functionList) {
                String define = function.getDefine();
                BudgetDefine budgetDefine = gson.fromJson(define, BudgetDefine.class);
                EnergyItemInfoVo energyItemInfoVo = new EnergyItemInfoVo();
                energyItemInfoVo.setItemId(budgetDefine.getItemId());
                energyItemInfoVo.setModelCode(budgetDefine.getModelCode());
                List<EnergyItemDataVo> dataList = funcIdDateAndDataMap.get(function.getId());
                energyItemInfoVo.setDataList(dataList == null ? new ArrayList<>() : dataList);
                resultList.add(energyItemInfoVo);
            }
            return resultList;
        } catch (Exception e) {
            throw new RuntimeException("查询分项数据异常", e);
        }
    }

    @Override
    public void saveEnergyItemData(SaveEnergyItemDataDTO saveEnergyItemDataDTO) {
        try {
            // 初始化用户信息，本接口非前端调用，因此使用默认的用户信息
            PoemsContext.setContext(DataConstants.USER_ID, DataConstants.LOGIN_DEVICE, DataConstants.PD);
            List<EnergyItemInfoVo> energyItemInfoVoList = saveEnergyItemDataDTO.getDataList();
            Map<String, String> itemIdFunctionIdMap = getFunctionIdMap(saveEnergyItemDataDTO);
            for (EnergyItemInfoVo energyItemInfoVo : energyItemInfoVoList) {
                // 根据itemId获取指标id
                String functionId = itemIdFunctionIdMap.get(energyItemInfoVo.getItemId());
                List<EnergyItemDataVo> energyItemDataVos = energyItemInfoVo.getDataList();
                List<Date> deleteDates = new ArrayList<>();
                List<ManualFillingData> manualFillingDataList = new ArrayList<>();
                for (EnergyItemDataVo energyItemDataVo : energyItemDataVos) {
                    ManualFillingData manualFillingData = new ManualFillingData();
                    manualFillingData.setFunctionId(functionId);
                    manualFillingData.setProjectId(saveEnergyItemDataDTO.getProjectId());
                    manualFillingData.setDataTime(DateUtils.str2Date(energyItemDataVo.getTime(), DateUtils.SDF_SECOND));
                    manualFillingData.setType(saveEnergyItemDataDTO.getFunctionType());
                    manualFillingData.setValue(energyItemDataVo.getValue());
                    manualFillingDataList.add(manualFillingData);

                    deleteDates.add(manualFillingData.getDataTime());
                }

                // 删除之前的记录
                manualFillingDataService.batchDelete(functionId, saveEnergyItemDataDTO.getProjectId(), deleteDates);
                manualFillingDataService.batchSave(manualFillingDataList);
            }
        } catch (Exception e) {
            throw new IllegalArgumentException("保存分项录入数据异常", e);
        }
    }

    /**
     * 获取要保存的能耗数据中itemId与指标id的映射关系
     *
     * @param saveEnergyItemDataDTO 要保存的能耗数据
     * @return 要保存的能耗数据中itemId与指标id的映射关系
     * @author lixing
     * @version V1.0 2021/7/29 10:51 上午
     */
    private Map<String, String> getFunctionIdMap(SaveEnergyItemDataDTO saveEnergyItemDataDTO) {
        Map<String, Object> defineMap = Maps.newConcurrentMap();
        defineMap.put("energyType", saveEnergyItemDataDTO.getEnergyType());
        List<Function> functionList = functionService.queryByDefine(
                FunctionDataSourceEnum.manualFilling.name(), defineMap);
        if (CollectionUtils.isEmpty(functionList)) {
            return new HashMap<>();
        }
        Map<String, String> resultMap = new HashMap<>();
        Gson gson = new Gson();
        for (Function function : functionList) {
            BudgetDefine budgetDefine = gson.fromJson(function.getDefine(), BudgetDefine.class);
            resultMap.put(budgetDefine.getItemId(), function.getId());
        }
        return resultMap;
    }

}
