package com.persagy.apm.report.dependencies.saasweb.service.impl;

import com.google.common.collect.Maps;
import com.persagy.apm.common.model.dto.PoemsFeignBaseDTO;
import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.report.userconfig.reporttype.service.IReportTypeService;
import com.persagy.apm.report.dependencies.saasweb.constants.SaaswebConstants;
import com.persagy.apm.report.dependencies.saasweb.constants.enums.AreaTypeEnum;
import com.persagy.apm.report.dependencies.saasweb.model.ConvertSaasWebTool;
import com.persagy.apm.report.dependencies.saasweb.model.dto.QueryByProjectIdDTO;
import com.persagy.apm.report.dependencies.saasweb.model.dto.QueryPartitionDTO;
import com.persagy.apm.report.dependencies.saasweb.model.dto.QueryProjectDTO;
import com.persagy.apm.report.dependencies.saasweb.model.vo.*;
import com.persagy.apm.report.dependencies.saasweb.service.ISaasWebService;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.common.utils.DateUtils;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lixing
 * @version V1.0 2021/5/21 10:51 上午
 **/
@Api(value = "运维平台service实现类")
@Service
@Slf4j
public class SaasWebServiceImpl implements ISaasWebService {
    @Autowired
    SaasWebClientWrapper saasWebClientWrapper;
    @Autowired
    IReportTypeService reportTypeService;

    @Override
    public String getProjectLocalId(String projectId) {
        SimpleProjectVO simpleProjectVO = getSimpleProjectInfo(projectId);
        if (simpleProjectVO != null) {
            return simpleProjectVO.getProjectLocalID();
        }
        return null;
    }

    @Override
    public ReportProjectVO getReportProjectInfo(String projectId) {
        ReportProjectVO reportProjectVO = new ReportProjectVO();
        reportProjectVO.setProjectId(projectId);

        SimpleProjectVO simpleProjectVO = getSimpleProjectInfo(projectId);

        if (simpleProjectVO != null) {
            reportProjectVO.setProjectLocalName(simpleProjectVO.getProjectLocalName());
            reportProjectVO.setFunctionType(simpleProjectVO.getFunctionType());
            reportProjectVO.setProjectLocalID(simpleProjectVO.getProjectLocalID());
            // 获取项目下建筑的面积总和
            ReportProjectVO projectAreas = getProjectAreas(projectId);
            reportProjectVO.setCommercialArea(projectAreas.getCommercialArea());
            reportProjectVO.setUndergroundGarageArea(projectAreas.getUndergroundGarageArea());
            reportProjectVO.setTotalBuildingArea(projectAreas.getTotalBuildingArea());
            reportProjectVO.setPublicArea(projectAreas.getPublicArea());
        }

        return reportProjectVO;
    }

    @Override
    public SimpleProjectVO getSimpleProjectInfo(String projectId) {
        if (StringUtils.isBlank(projectId)) {
            return null;
        }
        QueryByProjectIdDTO queryByProjectIdDTO = new QueryByProjectIdDTO();
        queryByProjectIdDTO.setProjectId(projectId);
        // 获取项目信息
        PoemsFeignResponse<SimpleProjectVO> projectInfoResponse = saasWebClientWrapper.getProjectById(queryByProjectIdDTO);
        List<SimpleProjectVO> projectInfos = projectInfoResponse.getContent();
        SimpleProjectVO simpleProjectVO = null;
        if (!CollectionUtils.isEmpty(projectInfos)) {
            simpleProjectVO = projectInfos.get(0);
        }
        return simpleProjectVO;
    }

    /**
     * 获取项目的面积信息
     *
     * @param projectId 项目id
     * @return 项目的面积信息
     * @author lixing
     * @version V1.0 2021/5/21 11:43 上午
     */
    private ReportProjectVO getProjectAreas(String projectId) {
        ReportProjectVO reportProjectVO = new ReportProjectVO();

        QueryByProjectIdDTO queryByProjectIdDTO = new QueryByProjectIdDTO();
        queryByProjectIdDTO.setProjectId(projectId);

        // 获取项目下的建筑信息
        PoemsFeignResponse<BuildingInfoVO> buildsResponse = saasWebClientWrapper.listBuildingsByProjectId(queryByProjectIdDTO);
        List<BuildingInfoVO> buildingInfoVOList = buildsResponse.getContent();
        // 统计项目下建筑总的商业面积，地下停车场面积，总面积，公区面积
        if (!CollectionUtils.isEmpty(buildingInfoVOList)) {
            List<String> commercialAreaList = buildingInfoVOList.stream().map(
                    BuildingInfoVO::getCommercialArea).collect(Collectors.toList());
            List<String> undergroundGarageAreaList = buildingInfoVOList.stream().map(
                    BuildingInfoVO::getUndergroundGarageArea).collect(Collectors.toList());
            List<String> totalAreaList = buildingInfoVOList.stream().map(
                    BuildingInfoVO::getTotalArea).collect(Collectors.toList());
            List<String> publicAreaAreaList = buildingInfoVOList.stream().map(
                    BuildingInfoVO::getPublicArea).collect(Collectors.toList());

            reportProjectVO.setCommercialArea(DataUtils.getSum(commercialAreaList));
            reportProjectVO.setUndergroundGarageArea(DataUtils.getSum(undergroundGarageAreaList));
            reportProjectVO.setTotalBuildingArea(DataUtils.getSum(totalAreaList));
            reportProjectVO.setPublicArea(DataUtils.getSum(publicAreaAreaList));
        }

        return reportProjectVO;
    }

    @Override
    public ReportAreaVO getReportAreaInfo(String areaCode, String buildingType) {
        ReportAreaVO reportAreaVO = getReportAreaBaseInfo(areaCode);

        List<PoemsProjectVO> projectsByAreaAndBuildingType = getProjectsByAreaAndBuildingType(areaCode, buildingType);

        ReportAreaVO reportAreaAreaInfo = getReportAreaAreaInfo(projectsByAreaAndBuildingType);
        reportAreaVO.setUndergroundGarageArea(reportAreaAreaInfo.getUndergroundGarageArea());
        reportAreaVO.setCommercialArea(reportAreaAreaInfo.getCommercialArea());
        return reportAreaVO;
    }

    /**
     * 获取分区的面积信息
     *
     * @param projects 分区下要统计的项目
     * @return 这些项目的面积信息
     * @author lixing
     * @version V1.0 2021/5/31 6:19 下午
     */
    private ReportAreaVO getReportAreaAreaInfo(List<PoemsProjectVO> projects) {
        ReportAreaVO reportAreaVO = new ReportAreaVO();
        // 统计分区下总的商业面积，地下停车场面积
        BigDecimal totalCommercialArea = new BigDecimal("0");
        BigDecimal totalUndergroundGarageArea = new BigDecimal("0");
        if (!CollectionUtils.isEmpty(projects)) {
            for (PoemsProjectVO poemsProjectVO : projects) {
                // 获取项目下建筑的面积总和
                ReportProjectVO projectAreas = getProjectAreas(poemsProjectVO.getProjectId());
                if (StringUtils.isNotBlank(projectAreas.getCommercialArea())) {
                    totalCommercialArea = totalCommercialArea.add(new BigDecimal(projectAreas.getCommercialArea()));
                }
                if (StringUtils.isNotBlank(projectAreas.getUndergroundGarageArea())) {
                    totalUndergroundGarageArea = totalUndergroundGarageArea.add(new BigDecimal(projectAreas.getUndergroundGarageArea()));
                }
            }
        }
        reportAreaVO.setCommercialArea(totalCommercialArea.toString());
        reportAreaVO.setUndergroundGarageArea(totalUndergroundGarageArea.toString());
        return reportAreaVO;
    }

    @Override
    public ReportAreaVO getReportAreaBaseInfo(String areaCode) {
        ReportAreaVO reportAreaVO = new ReportAreaVO();
        // 查询管理分区信息
        QueryPartitionDTO queryPartitionDTO = new QueryPartitionDTO();
        queryPartitionDTO.setManagePartitionIds(Lists.newArrayList(areaCode));
        PoemsFeignResponse<PoemsPartitionVO> partitionsResponse = saasWebClientWrapper.queryPartitionsByIds(queryPartitionDTO);
        List<PoemsPartitionVO> partitionVOList = partitionsResponse.getContent();
        if (!CollectionUtils.isEmpty(partitionVOList)) {
            PoemsPartitionVO poemsPartitionVO = partitionVOList.get(0);
            reportAreaVO.setAreaId(areaCode);
            reportAreaVO.setAreaName(poemsPartitionVO.getManagePartitionName());
        }
        return reportAreaVO;
    }

    @Override
    public List<PoemsProjectVO> getProjectsByArea(String areaCode) {
        // 获取管理分区下的项目
        QueryProjectDTO queryProjectDTO = new QueryProjectDTO();
        queryProjectDTO.setGroupManageZone(areaCode);
        PoemsFeignResponse<PoemsProjectVO> projectsResponse = saasWebClientWrapper.queryProjects(queryProjectDTO);
        return projectsResponse.getContent();
    }

    @Override
    public List<PoemsProjectVO> getAllProjects() {
        // 获取管理分区下的项目
        QueryProjectDTO queryProjectDTO = new QueryProjectDTO();
        PoemsFeignResponse<PoemsProjectVO> projectsResponse = saasWebClientWrapper.queryProjects(queryProjectDTO);
        return projectsResponse.getContent();
    }

    @Override
    public List<PoemsProjectVO> getProjectsByClimateAndBuildingType(String climateZoneCode, String buildingTypeCode) {
        List<PoemsProjectVO> allProjects = getAllProjects();
        List<String> buildingTypeCodes = getRelatedBuildingTypes(buildingTypeCode);

        if (CollectionUtils.isEmpty(buildingTypeCodes) || StringUtils.isBlank(climateZoneCode)) {
            return Lists.newArrayList();
        }
        return allProjects.stream().
                filter(
                        poemsProjectVO -> satisfyBuildingTypeAndClimateZone(
                                buildingTypeCodes, climateZoneCode, poemsProjectVO)
                ).collect(Collectors.toList());
    }

    @Override
    public List<PoemsProjectVO> getProjectsByAreaAndBuildingType(String areaCode, String buildingType) {
        return getProjectsByAreaAndBuildingTypes(areaCode, Lists.newArrayList(buildingType));
    }

    @Override
    public List<PoemsProjectVO> getProjectsByAreaAndBuildingTypes(String areaCode, List<String> buildingTypes) {
        if (StringUtils.isBlank(areaCode) || CollectionUtils.isEmpty(buildingTypes)) {
            throw new IllegalArgumentException("根据业态和区域过滤项目，业态或区域为空");
        }
        List<PoemsProjectVO> projectsByArea = getProjectsByArea(areaCode);
        if (CollectionUtils.isEmpty(projectsByArea)) {
            return Lists.newArrayList();
        }

        List<String> relatedBuildingTypes = new ArrayList<>();
        for (String buildingType : buildingTypes) {
            relatedBuildingTypes.addAll(getRelatedBuildingTypes(buildingType));
        }


        if (CollectionUtils.isEmpty(relatedBuildingTypes)) {
            return Lists.newArrayList();
        }
        return projectsByArea.stream().
                filter(
                        poemsProjectVO -> satisfyBuildingTypeAndClimateZone(
                                relatedBuildingTypes, null, poemsProjectVO)
                ).collect(Collectors.toList());
    }

    @Override
    public List<PoemsProjectVO> getProjectsByBuildingType(String buildingType) {
        List<PoemsProjectVO> allProjects = getAllProjects();
        List<String> buildingTypeCodes = getRelatedBuildingTypes(buildingType);

        if (CollectionUtils.isEmpty(buildingTypeCodes)) {
            return Lists.newArrayList();
        }
        return allProjects.stream().
                filter(
                        poemsProjectVO -> satisfyBuildingTypeAndClimateZone(
                                buildingTypeCodes, null, poemsProjectVO)
                ).collect(Collectors.toList());
    }

    /**
     * 判断项目是否满足业态和气候区筛选条件
     *
     * @param allBuildingTypes 所有用来筛选的业态
     * @param climateZoneCode  用来筛选的气候区
     * @param projectVO        PoemsProjectVO对象
     * @return 是否满足条件
     * @author lixing
     * @version V1.0 2021/6/15 5:12 下午
     */
    private boolean satisfyBuildingTypeAndClimateZone(
            List<String> allBuildingTypes, String climateZoneCode, PoemsProjectVO projectVO) {


        if (projectVO == null) {
            return false;
        }

        boolean result = true;
        // 如果要过滤的建筑类型不为空，根据建筑类型过滤
        if (!CollectionUtils.isEmpty(allBuildingTypes)) {
            result = StringUtils.isNotBlank(projectVO.getFunctionType()) &&
                    allBuildingTypes.contains(projectVO.getFunctionType());
        }

        // 如果要过滤的气候区不为空，根据气候区过滤
        if (StringUtils.isNotBlank(climateZoneCode)) {
            result = result && climateZoneCode.equals(projectVO.getClimateZone());
        }

        return result;
    }

    @Override
    public List<SimpleProjectVO> getAvailableProjects(String buildingType) {
        // 获取所有关联的业态
        List<String> buildingTypes = getRelatedBuildingTypes(buildingType);

        // 获取用户可选的所有项目
        return getProjectsByUserAndBuildingTypes(buildingTypes);
    }

    @Override
    public List<SimpleProjectVO> getAvailableProjects(List<String> buildingTypes) {
        if (CollectionUtils.isEmpty(buildingTypes)) {
            return null;
        }

        // 获取所有关联的业态
        List<String> relatedBuildingTypes = new ArrayList<>();
        for (String buildingType : buildingTypes) {
            relatedBuildingTypes.addAll(getRelatedBuildingTypes(buildingType));
        }

        return getProjectsByUserAndBuildingTypes(relatedBuildingTypes);
    }

    /**
     * 根据用户权限和业态过滤项目
     *
     * @param buildingTypes 需要过滤的业态（不包含子业态），可为空
     * @return 在指定业态下用户拥有的项目列表
     * @author lixing
     * @version V1.0 2021/7/16 12:06 下午
     */
    private List<SimpleProjectVO> getProjectsByUserAndBuildingTypes(List<String> buildingTypes) {
        List<SimpleProjectVO> result = Lists.newArrayList();
        // 获取用户可选的所有项目
        PoemsFeignBaseDTO poemsFeignBaseDTO = new PoemsFeignBaseDTO();
        poemsFeignBaseDTO.setDefaultValue();
        PoemsFeignResponse<UserProjectsInfoVO> currentUserProjectsInfosResponse = saasWebClientWrapper.
                getCurrentUserProjectsInfo(poemsFeignBaseDTO);
        List<UserProjectsInfoVO> currentUserProjectsInfos = currentUserProjectsInfosResponse.getContent();
        if (!CollectionUtils.isEmpty(currentUserProjectsInfos)) {
            UserProjectsInfoVO userProjectsInfoVO = currentUserProjectsInfos.get(0);
            if (userProjectsInfoVO != null) {
                List<PoemsProjectVO> projects = userProjectsInfoVO.getProjects();
                projects = projects.stream().filter(
                        poemsProjectVO -> satisfyBuildingTypeAndClimateZone(
                                buildingTypes, null, poemsProjectVO)
                ).collect(Collectors.toList());
                List<SimpleProjectVO> simpleProjectVOList = ConvertSaasWebTool.INSTANCE.convert2SimpleProjectVOList(projects);
                result.addAll(simpleProjectVOList);
            }
        }
        return result;
    }

    @Override
    public List<String> getRelatedBuildingTypes(String buildingType) {
        if (StringUtils.isBlank(buildingType)) {
            return Lists.newArrayList();
        }
        List<FunctionTypeVO> sonBuildingTypes = findSonBuildingTypes(buildingType);
        List<String> buildingTypes = Lists.newArrayList(buildingType);
        functionTypeTree2List(sonBuildingTypes, buildingTypes);
        return buildingTypes;
    }

    @Override
    public List<String> getRelatedBuildingTypes(List<String> buildingTypes) {
        if (CollectionUtils.isEmpty(buildingTypes)) {
            return Lists.newArrayList();
        }
        Set<String> resultSet = new HashSet<>();
        for (String buildingType : buildingTypes) {
            resultSet.addAll(getRelatedBuildingTypes(buildingType));
        }
        return new ArrayList<>(resultSet);
    }

    /**
     * 项目功能类型树转换为项目功能编码列表
     *
     * @param functionTypeVOList 项目功能类型树
     * @param result             存储项目功能编码列表的对象
     * @author lixing
     * @version V1.0 2021/5/21 2:42 下午
     */
    private void functionTypeTree2List(List<FunctionTypeVO> functionTypeVOList, List<String> result) {
        if (result == null) {
            result = Lists.newArrayList();
        }
        if (CollectionUtils.isEmpty(functionTypeVOList)) {
            return;
        }
        for (FunctionTypeVO functionTypeVO : functionTypeVOList) {
            result.add(functionTypeVO.getCode());
            functionTypeTree2List(functionTypeVO.getContents(), result);
        }
    }

    /**
     * 获取项目功能类型的子类型
     *
     * @param buildingType 要获取子类型的项目功能类型
     * @return 子类型列表树
     * @author lixing
     * @version V1.0 2021/5/21 2:40 下午
     */
    private List<FunctionTypeVO> findSonBuildingTypes(String buildingType) {
        if (StringUtils.isBlank(buildingType)) {
            return Lists.newArrayList();
        }
        // 获取业态下所有的子业态
        PoemsFeignBaseDTO poemsFeignBaseDTO = new PoemsFeignBaseDTO();
        poemsFeignBaseDTO.setDefaultValue();
        PoemsFeignResponse<FunctionTypeVO> functionTypesResponse = saasWebClientWrapper.getAllProjectFunctionTypes(poemsFeignBaseDTO);
        List<FunctionTypeVO> functionTypeVOList = functionTypesResponse.getContent();
        return recursiveFindSonBuildingTypes(buildingType, functionTypeVOList);
    }

    /**
     * 递归获取项目功能类型的子类型
     *
     * @param buildingType       要获取子类型的项目功能类型
     * @param functionTypeVOList 所有的项目功能类型
     * @return 子类型列表树
     * @author lixing
     * @version V1.0 2021/5/21 2:40 下午
     */
    private List<FunctionTypeVO> recursiveFindSonBuildingTypes(String buildingType, List<FunctionTypeVO> functionTypeVOList) {
        if (!CollectionUtils.isEmpty(functionTypeVOList)) {
            for (FunctionTypeVO functionTypeVO : functionTypeVOList) {
                if (functionTypeVO.getCode().equals(buildingType)) {
                    return functionTypeVO.getContents();
                }
                List<FunctionTypeVO> sonBuildingTypes = recursiveFindSonBuildingTypes(buildingType, functionTypeVO.getContents());
                if (!CollectionUtils.isEmpty(sonBuildingTypes)) {
                    return sonBuildingTypes;
                }
            }
        }
        return Lists.newArrayList();
    }

    @Override
    public List<PartitionVO> getAvailableAreaTree(String buildingType) {
        // 遍历用户所有可选分区树，标记分区父节点及分区拥有的项目
        List<FilterPartitionVO> partitions = partitionProjectTree2FilterPartitionList();
        // 筛选分区
        Map<String, FilterPartitionVO> filterPartitionMap = filterPartitionsByBuildingType(partitions, buildingType);
        // 将筛选后的分区重新组装成树
        return filterPartitionMap2Tree(filterPartitionMap);
    }

//    @Override
//    public List<PartitionVO> getAvailableAreaTree(List<String> buildingTypes) {
//        // 遍历用户所有可选分区树，标记分区父节点及分区拥有的项目
//        List<FilterPartitionVO> partitions = partitionProjectTree2FilterPartitionList();
//        // 筛选分区
//        Map<String, FilterPartitionVO> filterPartitionMap = filterPartitionsByBuildingTypes(partitions, buildingTypes);
//        // 将筛选后的分区重新组装成树
//        return filterPartitionMap2Tree(filterPartitionMap);
//    }

    @Override
    public List<PartitionVO> getAvailableAreaTree(List<String> buildingTypes) {
        Date startTime = new Date();
        // 遍历用户所有可选分区树，标记分区父节点及分区拥有的项目
        List<FilterPartitionVO> partitions = partitionProjectTree2FilterPartitionList();
        System.out.println("timeDuration1:" + DateUtils.getDateDurationMs(startTime, new Date()));
        // 筛选分区
        Map<String, FilterPartitionVO> filterPartitionMap = filterPartitionsByBuildingTypes(partitions, buildingTypes);
        System.out.println("timeDuration2:" + DateUtils.getDateDurationMs(startTime, new Date()));
        // 将筛选后的分区重新组装成树
        List<PartitionVO> partitionVOS = filterPartitionMap2Tree(filterPartitionMap);
        System.out.println("timeDuration3:" + DateUtils.getDateDurationMs(startTime, new Date()));
        return partitionVOS;
    }

    @Override
    public List<PartitionVO> getSonAreas(List<String> buildingTypes, String areaId) {
        if (StringUtils.isBlank(areaId)) {
            return null;
        }
        // 遍历用户所有可选分区树，标记分区父节点及分区拥有的项目
        List<FilterPartitionVO> partitions = partitionProjectTree2FilterPartitionList();
        // 筛选分区
        Map<String, FilterPartitionVO> filterPartitionMap = filterPartitionsByBuildingTypes(partitions, buildingTypes);
        if (CollectionUtils.isEmpty(filterPartitionMap)) {
            return null;
        }
        return filterPartitionMap.values().stream().filter(
                filterPartition -> areaId.equals(filterPartition.getParentId())
        ).map(ConvertSaasWebTool.INSTANCE::convert2PartitionVO).collect(Collectors.toList());
    }

    @Override
    public List<String> getAvailableAreaIds(String buildingType) {
        // 遍历用户所有可选分区树，标记分区父节点及分区拥有的项目
        List<FilterPartitionVO> partitions = partitionProjectTree2FilterPartitionList();
        // 筛选分区
        Map<String, FilterPartitionVO> filterPartitionMap = filterPartitionsByBuildingType(partitions, buildingType);
        if (CollectionUtils.isEmpty(filterPartitionMap)) {
            return Lists.newArrayList();
        } else {
            return Lists.newArrayList(filterPartitionMap.keySet());
        }
    }

    @Override
    public List<String> getAvailableAreaIds(List<String> buildingTypes) {
        // 遍历用户所有可选分区树，标记分区父节点及分区拥有的项目
        List<FilterPartitionVO> partitions = partitionProjectTree2FilterPartitionList();
        // 筛选分区
        Map<String, FilterPartitionVO> filterPartitionMap = filterPartitionsByBuildingTypes(partitions, buildingTypes);
        if (CollectionUtils.isEmpty(filterPartitionMap)) {
            return Lists.newArrayList();
        } else {
            return Lists.newArrayList(filterPartitionMap.keySet());
        }
    }

    @Override
    public List<PartitionProjectVO> getAvailableProjectTree(List<String> buildingTypes) {
        // 查询用户权限范围内的分区和项目
        List<PartitionProjectVO> availableProjectTree = getAvailableProjectTree();
        if (CollectionUtils.isEmpty(availableProjectTree)) {
            return Lists.newArrayList();
        }

        // 根据建筑业态筛选用户权限范围内的项目
        List<SimpleProjectVO> availableProjects = getAvailableProjects(buildingTypes);
        if (CollectionUtils.isEmpty(availableProjects)) {
            return Lists.newArrayList();
        }

        /* 根据用户可选项目筛选分区 */
        Set<String> availableProjectIds = availableProjects.stream().
                map(SimpleProjectVO::getProjectId).collect(Collectors.toSet());
        // 遍历用户所有可选分区树，标记分区父节点及分区拥有的项目
        List<FilterPartitionVO> partitions = partitionProjectTree2FilterPartitionList();
        Map<String, FilterPartitionVO> filterPartitionMap = filterPartitions(partitions, availableProjectIds);
        if (CollectionUtils.isEmpty(filterPartitionMap)) {
            return Lists.newArrayList();
        }

        Map<String, String> filterProjectMap = availableProjects.stream().collect(
                Collectors.toMap(SimpleProjectVO::getProjectId, SimpleProjectVO::getProjectLocalName, (k1, k2) -> k1));

        return cascadeFilterAvailablePartitionAndProject(
                availableProjectTree, filterPartitionMap, filterProjectMap);
    }

    /**
     * 递归过滤用户可用的分区和项目
     *
     * @param treeBeforeFilter      需要过滤的树
     * @param availablePartitionMap 可用分区map
     * @param availableProjectMap   可用项目map
     * @return 用户可用的分区项目树
     * @author lixing
     * @version V1.0 2021/6/18 11:12 上午
     */
    private List<PartitionProjectVO> cascadeFilterAvailablePartitionAndProject(
            List<PartitionProjectVO> treeBeforeFilter, Map<String, FilterPartitionVO> availablePartitionMap,
            Map<String, String> availableProjectMap) {
        List<PartitionProjectVO> filteredTree = Lists.newArrayList(treeBeforeFilter);
        for (PartitionProjectVO partitionProjectVO : treeBeforeFilter) {
            // 节点为项目，但不满足业态条件，从树中剔除
            if (SaaswebConstants.PROJECT_FLAG.equals(partitionProjectVO.getType()) &&
                    availableProjectMap.get(partitionProjectVO.getId()) == null) {
                filteredTree.remove(partitionProjectVO);
            } else if (SaaswebConstants.AREA_FLAG.equals(partitionProjectVO.getType()) &&
                    availablePartitionMap.get(partitionProjectVO.getId()) == null) {
                // 节点为分区，但不满足业态条件，从树中剔除
                filteredTree.remove(partitionProjectVO);
            }

        }
        if (CollectionUtils.isEmpty(filteredTree)) {
            return filteredTree;
        }
        for (PartitionProjectVO partitionProjectVO : filteredTree) {
            // 递归处理下级节点
            List<PartitionProjectVO> sonNodes = partitionProjectVO.getContents();
            if (!CollectionUtils.isEmpty(sonNodes)) {
                List<PartitionProjectVO> filterSonNodes = cascadeFilterAvailablePartitionAndProject(
                        sonNodes, availablePartitionMap, availableProjectMap);
                partitionProjectVO.setContents(filterSonNodes);
            }
        }
        return filteredTree;
    }

    /**
     * 当前用户拥有的分区项目树
     *
     * @return 分区项目树
     * @author lixing
     * @version V1.0 2021/6/17 2:48 下午
     */
    private List<PartitionProjectVO> getAvailableProjectTree() {
        PoemsFeignBaseDTO poemsFeignBaseDTO = new PoemsFeignBaseDTO();
        poemsFeignBaseDTO.setDefaultValue();
        PoemsFeignResponse<UserPartitionProjectsInfoVO> partitionsResponse = saasWebClientWrapper.getCurrentUserPartitionProjectsInfo(poemsFeignBaseDTO);
        List<UserPartitionProjectsInfoVO> userPartitionProjects = partitionsResponse.getContent();
        if (CollectionUtils.isEmpty(userPartitionProjects)) {
            return Lists.newArrayList();
        }
        return userPartitionProjects.get(0).getPartitionProjects();
    }

    /**
     * 将筛选后的分区组装成树
     *
     * @param filterPartitionMap 筛选后的分区
     * @return 树根节点列表
     * @author lixing
     * @version V1.0 2021/5/21 6:07 下午
     */
    private List<PartitionVO> filterPartitionMap2Tree(Map<String, FilterPartitionVO> filterPartitionMap) {
        if (filterPartitionMap.isEmpty()) {
            return Lists.newArrayList();
        }
        List<PartitionVO> result = new ArrayList<>();
        Map<String, List<FilterPartitionVO>> parentIdMap = new HashMap<>();
        for (String areaId : filterPartitionMap.keySet()) {
            FilterPartitionVO filterPartitionVO = filterPartitionMap.get(areaId);
            String parentId = filterPartitionVO.getParentId();
            if (StringUtils.isNotBlank(parentId) && filterPartitionMap.containsKey(parentId)) {
                List<FilterPartitionVO> filterPartitionVOList = parentIdMap.get(parentId);
                if (filterPartitionVOList == null) {
                    filterPartitionVOList = new ArrayList<>();
                }
                filterPartitionVOList.add(filterPartitionVO);
                parentIdMap.put(parentId, filterPartitionVOList);
            } else {
                PartitionVO partitionVO = ConvertSaasWebTool.INSTANCE.convert2PartitionVO(filterPartitionVO);
                result.add(partitionVO);
            }
        }

        // 给根节点添加子节点
        cascadeSetPartitionChildren(result, parentIdMap);

        return result;
    }

    /**
     * 级联给区域的子节点赋值
     *
     * @param result      需要赋值的节点
     * @param parentIdMap 区域父节点id -> 区域列表
     * @author lixing
     * @version V1.0 2021/5/28 5:51 下午
     */
    private void cascadeSetPartitionChildren(List<PartitionVO> result, Map<String, List<FilterPartitionVO>> parentIdMap) {
        if (CollectionUtils.isEmpty(result)) {
            return;
        }
        for (PartitionVO partitionVO : result) {
            List<FilterPartitionVO> filterPartitionVOList = parentIdMap.get(partitionVO.getAreaId());
            List<PartitionVO> partitionVOList = ConvertSaasWebTool.INSTANCE.convert2PartitionVOList(filterPartitionVOList);
            partitionVO.setChildren(partitionVOList);
            cascadeSetPartitionChildren(partitionVOList, parentIdMap);
        }
    }

    /**
     * 遍历用户所有可选分区树，标记分区父节点及分区拥有的项目
     *
     * @return 处理后的分区列表
     * @author lixing
     * @version V1.0 2021/5/21 3:59 下午
     */
    private List<FilterPartitionVO> partitionProjectTree2FilterPartitionList() {
        // 获取用户所有可选的分区信息
        List<PartitionProjectVO> partitionProjects = getAvailableProjectTree();

        // 筛选出分区列表
        List<PartitionProjectVO> partitions = new ArrayList<>();
        if (CollectionUtils.isEmpty(partitionProjects)) {
            return Lists.newArrayList();
        }
        for (PartitionProjectVO partitionProject : partitionProjects) {
            String projectFlag = "0";
            if (!projectFlag.equals(partitionProject.getType())) {
                partitions.add(partitionProject);
            }
            getPartitionsInPartitionProjects(partitionProject, partitions);
        }

        // 获取分区下的项目和分区的父节点
        List<FilterPartitionVO> result = Lists.newArrayList();
        for (PartitionProjectVO partition : partitions) {
            // 获取每个分区下拥有的项目
            List<SimpleProjectVO> projects = Lists.newArrayList();
            getProjectsUnderPartition(partition, projects);

            FilterPartitionVO filterPartitionVO = new FilterPartitionVO();
            filterPartitionVO.setAreaId(partition.getId());
            filterPartitionVO.setAreaName(partition.getName());
            filterPartitionVO.setParentId(partition.getParentId());
            filterPartitionVO.setProjects(projects);

            result.add(filterPartitionVO);
        }

        return result;
    }

    /**
     * 获取分区项目里所有的分区
     *
     * @param partitionProject 分区项目对象
     * @param result           存储分区的对象
     * @author lixing
     * @version V1.0 2021/5/21 5:42 下午
     */
    private void getPartitionsInPartitionProjects(PartitionProjectVO partitionProject, List<PartitionProjectVO> result) {
        String projectFlag = "0";
        if (projectFlag.equals(partitionProject.getType())) {
            return;
        }
        List<PartitionProjectVO> contents = partitionProject.getContents();
        for (PartitionProjectVO content : contents) {
            if (!projectFlag.equals(content.getType())) {
                content.setParentId(partitionProject.getId());
                result.add(content);
            }
            getPartitionsInPartitionProjects(content, result);
        }
    }

    /**
     * 获取分区下的项目
     *
     * @param partitionProject 分区项目对象
     * @param result           存储分区下项目的列表
     * @author lixing
     * @version V1.0 2021/5/21 5:39 下午
     */
    private void getProjectsUnderPartition(PartitionProjectVO partitionProject, List<SimpleProjectVO> result) {
        String projectFlag = "0";
        if (projectFlag.equals(partitionProject.getType())) {
            return;
        }
        List<PartitionProjectVO> contents = partitionProject.getContents();
        for (PartitionProjectVO content : contents) {
            if (projectFlag.equals(content.getType())) {
                SimpleProjectVO simpleProjectVO = new SimpleProjectVO();
                simpleProjectVO.setProjectId(content.getId());
                simpleProjectVO.setProjectLocalName(content.getName());
                result.add(simpleProjectVO);
            } else {
                getProjectsUnderPartition(content, result);
            }
        }
    }

    /**
     * 根据项目，筛选分区
     *
     * @param partitions 用户所有可选分区列表
     * @param projectIds 用户可用项目id列表
     * @return 处理后的分区map 分区编码 -> 分区对象
     * @author lixing
     * @version V1.0 2021/5/21 3:59 下午
     */
    private Map<String, FilterPartitionVO> filterPartitions(
            List<FilterPartitionVO> partitions, Set<String> projectIds) {
        Map<String, FilterPartitionVO> result = Maps.newHashMap();

        if (CollectionUtils.isEmpty(partitions)) {
            return result;
        }

        for (FilterPartitionVO partition : partitions) {
            List<SimpleProjectVO> projects = partition.getProjects();
            if (CollectionUtils.isEmpty(projects)) {
                continue;
            }
            Set<String> projectIdsInArea = projects.stream().
                    map(SimpleProjectVO::getProjectId).collect(Collectors.toSet());
            // 取用户可选项目和分区下项目的交集，projectIds会被更新为两个集合的交集 projectIds是用户在分区下可选的项目
            projectIdsInArea.retainAll(projectIds);
            if (projectIdsInArea.size() > 0) {
                result.put(partition.getAreaId(), partition);
            }
        }
        return result;
    }

    /**
     * 根据业态，筛选分区
     *
     * @param partitions   用户所有可选分区列表
     * @param buildingTypes 要筛选的业态列表
     * @return 处理后的分区map 分区编码 -> 分区对象
     * @author lixing
     * @version V1.0 2021/5/21 3:59 下午
     */
    private Map<String, FilterPartitionVO> filterPartitionsByBuildingTypes(
            List<FilterPartitionVO> partitions, List<String> buildingTypes) {
        Map<String, FilterPartitionVO> result = Maps.newHashMap();

        if (CollectionUtils.isEmpty(partitions)) {
            return result;
        }
        // 获取用户在当前业态下可选的项目
        List<SimpleProjectVO> availableProjects = getAvailableProjects(buildingTypes);
        if (CollectionUtils.isEmpty(availableProjects)) {
            return result;
        }

        Set<String> availableProjectIds = availableProjects.stream().
                map(SimpleProjectVO::getProjectId).collect(Collectors.toSet());

        for (FilterPartitionVO partition : partitions) {
            List<SimpleProjectVO> projects = partition.getProjects();
            if (CollectionUtils.isEmpty(projects)) {
                continue;
            }
            Set<String> projectIds = projects.stream().
                    map(SimpleProjectVO::getProjectId).collect(Collectors.toSet());
            // 取用户可选项目和分区下项目的交集，projectIds会被更新为两个集合的交集 projectIds是用户在分区下可选的项目
            projectIds.retainAll(availableProjectIds);
            // 根据业态和分区过滤项目
            List<PoemsProjectVO> projectsByAreaAndBuildingType = getProjectsByAreaAndBuildingTypes(partition.getAreaId(), buildingTypes);
            Set<String> projectIdsByAreaAndBuildingType = projectsByAreaAndBuildingType.stream().
                    map(PoemsProjectVO::getProjectId).collect(Collectors.toSet());
            //            projectIdsByAreaAndBuildingType.retainAll(availableProjectIds);
            // 如果用户拥有的项目数和分区下的项目数不一致，则用户没有该分区的权限
            if (CollectionUtils.isEmpty(projectIds) || projectIdsByAreaAndBuildingType.size() != projectIds.size()) {
                continue;
            }
            partition.setProjects(
                    projects.stream().filter(
                            project -> projectIds.contains(project.getProjectId())).
                            collect(Collectors.toList()));
            result.put(partition.getAreaId(), partition);
        }
        return result;
    }

    /**
     * 根据业态，筛选分区
     *
     * @param partitions   用户所有可选分区列表
     * @param buildingType 要筛选的业态
     * @return 处理后的分区map 分区编码 -> 分区对象
     * @author lixing
     * @version V1.0 2021/5/21 3:59 下午
     */
    private Map<String, FilterPartitionVO> filterPartitionsByBuildingType(
            List<FilterPartitionVO> partitions, String buildingType) {
        return filterPartitionsByBuildingTypes(partitions, Lists.newArrayList(buildingType));
    }

    @Override
    public Double getTotalCommercialArea(List<String> projectIds) {
        return calculateProjectsArea(projectIds, AreaTypeEnum.COMMERCIAL);
    }

    @Override
    public Double getTotalPublicArea(List<String> projectIds) {
        return calculateProjectsArea(projectIds, AreaTypeEnum.PUBLIC_AREA);
    }

    @Override
    public Double getTotalBuildingArea(List<String> projectIds) {
        return calculateProjectsArea(projectIds, AreaTypeEnum.TOTAL);
    }

    @Override
    public Double getTotalUndergroundArea(List<String> projectIds) {
        return calculateProjectsArea(projectIds, AreaTypeEnum.UNDERGROUND_GARAGE);
    }

    /**
     * 计算项目的面积总和
     *
     * @param projectIds 要计算的项目id列表
     * @param areaType   面积类型
     * @return 计算结果
     * @author lixing
     * @version V1.0 2021/6/23 2:58 下午
     */
    private Double calculateProjectsArea(List<String> projectIds, AreaTypeEnum areaType) {
        if (CollectionUtils.isEmpty(projectIds)) {
            return null;
        }
        List<String> areaList = projectIds.stream().map(projectId -> {
            ReportProjectVO reportProjectInfo = getReportProjectInfo(projectId);
            if (reportProjectInfo == null) {
                return null;
            }
            switch (areaType) {
                case TOTAL:
                    return reportProjectInfo.getTotalBuildingArea();
                case PUBLIC_AREA:
                    return reportProjectInfo.getPublicArea();
                case UNDERGROUND_GARAGE:
                    return reportProjectInfo.getUndergroundGarageArea();
                case COMMERCIAL:
                    return reportProjectInfo.getCommercialArea();
                default:
                    return null;
            }
        }).collect(Collectors.toList());

        String resultStr = DataUtils.getSum(areaList);
        return StringUtils.isBlank(resultStr) ? null : Double.parseDouble(resultStr);
    }

    @Override
    public List<String> getComparableProjects(List<String> projectIds, Date month) {
        if (CollectionUtils.isEmpty(projectIds)) {
            return Lists.newArrayList();
        }
        List<String> result = new ArrayList<>();
        for (String projectId : projectIds) {
            SimpleProjectVO simpleProjectInfo = getSimpleProjectInfo(projectId);
            String businessOpeningTime = simpleProjectInfo.getBusinessOpeningTime();
            try {
                // 如果开业时间早于去年，添加到结果集
                if (isComparable(month, businessOpeningTime)) {
                    result.add(projectId);
                }
            } catch (Exception e) {
                throw new RuntimeException("时间转换异常", e);
            }
        }
        return result;
    }

    @Override
    public ReportAreaVO getCompareReportAreaInfo(String areaCode, String buildingType, Date reportMonth) {
        ReportAreaVO reportAreaVO = getReportAreaBaseInfo(areaCode);
        List<PoemsProjectVO> projectsByAreaAndBuildingType = getProjectsByAreaAndBuildingType(areaCode, buildingType);
        if (CollectionUtils.isEmpty(projectsByAreaAndBuildingType)) {
            return reportAreaVO;
        }
        List<PoemsProjectVO> comparableProjects = projectsByAreaAndBuildingType.stream().filter(poemsProjectVO -> {
            String businessOpeningTime = poemsProjectVO.getBusinessOpeningTime();
            try {
                return isComparable(reportMonth, businessOpeningTime);
            } catch (Exception e) {
                throw new RuntimeException("时间转换异常", e);
            }
        }).collect(Collectors.toList());
        ReportAreaVO reportAreaAreaInfo = getReportAreaAreaInfo(comparableProjects);
        reportAreaVO.setUndergroundGarageArea(reportAreaAreaInfo.getUndergroundGarageArea());
        reportAreaVO.setCommercialArea(reportAreaAreaInfo.getCommercialArea());
        return reportAreaVO;
    }

    /**
     * 如果开业时间早于报告月份上一年一月一日，项目为可同比项目
     *
     * @param reportMonth         报告月份
     * @param businessOpeningTime 开业时间
     * @return 是否是可同比项目
     * @author lixing
     * @version V1.0 2021/6/3 11:44 上午
     */
    private boolean isComparable(Date reportMonth, String businessOpeningTime) throws Exception {
        if (StringUtils.isBlank(businessOpeningTime)) {
            return false;
        }
        Date openDate = DateUtils.str2Date(businessOpeningTime, DateUtils.SDFDAY);
        Date firstDayOfLastYear = DateUtils.getFirstDayOfLastYear(reportMonth);
        // 如果开业时间早于去年，添加到结果集
        return DateUtils.compareDate(openDate, firstDayOfLastYear);
    }

    @Override
    public String getProjectClimateZone(String projectId) {
        SimpleProjectVO simpleProjectInfo = getSimpleProjectInfo(projectId);
        if (simpleProjectInfo != null) {
            return simpleProjectInfo.getClimateZone();
        }
        return null;
    }

    @Override
    public Map<String, List<String>> groupByFunctionType(List<SimpleProjectVO> projectVOList) {
        if (CollectionUtils.isEmpty(projectVOList)) {
            return null;
        }
        Map<String, List<String>> result = new HashMap<>();
        for (SimpleProjectVO simpleProjectVO : projectVOList) {
            String functionType = simpleProjectVO.getFunctionType();
            if (result.containsKey(functionType)) {
                List<String> projectIds = result.get(functionType);
                projectIds.add(simpleProjectVO.getProjectId());
            } else {
                List<String> projectIds = new ArrayList<>();
                projectIds.add(simpleProjectVO.getProjectId());
                result.put(functionType, projectIds);
            }
        }
        return result;
    }
}
