package com.persagy.apm.report.detail.paragraphvalue.service;

import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.paragraphvalue.model.*;
import com.persagy.apm.report.detail.paragraphvalue.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ReportParagraphValueItemVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 报告段落取值(ReportParagraphValue) service接口
 *
 * @author lixing
 * @version V1.0 2021-07-14 16:59:57
 */
public interface IReportParagraphValueService {

    /**
     * 创建报告段落取值
     *
     * @param addReportParagraphValueDTO addDTO
     * @return 报告段落取值主键
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    String createReportParagraphValue(AddReportParagraphValueDTO addReportParagraphValueDTO);

    /**
     * 报告段落取值详情
     *
     * @param id 主键
     * @return 报告段落取值do对象
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    ReportParagraphValue queryReportParagraphValueDetail(String id);

    /**
     * 更新报告段落取值
     *
     * @param updateReportParagraphValueDTO updateDTO
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    void updateReportParagraphValue(UpdateReportParagraphValueDTO updateReportParagraphValueDTO);

    /**
     * 删除报告段落取值
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    void deleteReportParagraphValue(String id);

    /**
     * 查询报告段落取值
     *
     * @param queryReportParagraphValueDTO queryDTO
     * @return List<ReportParagraphValue>
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    List<ReportParagraphValue> queryReportParagraphValueList(QueryReportParagraphValueDTO queryReportParagraphValueDTO);

    /**
     * 分页查询报告段落取值
     *
     * @param pageQueryReportParagraphValueDTO pageQueryDTO
     * @return IPage<ReportParagraphValue>
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    IPage<ReportParagraphValue> pageQueryReportParagraphValue(PageQueryReportParagraphValueDTO pageQueryReportParagraphValueDTO);

    /**
     * 获取段落的取值
     *
     * @param reportOutlineId 报告id
     * @param paragraphId 段落id
     * @return 段落的取值
     * @author lixing
     * @version V1.0 2021/7/15 5:57 下午
     */
    ReportParagraphValueItemVO getParagraphValue(String reportOutlineId, String paragraphId);

    /**
     * 删除报告下的段落
     *
     * @param reportOutlineId 报告概要id
     * @author lixing
     * @version V1.0 2021/7/20 10:08 上午
     */
    void deleteByReport(String reportOutlineId);

    /**
     * 更新报告补充说明
     *
     * @param reportOutlineId 报告概要id
     * @param paragraphId 段落id
     * @param supplement 补充说明
     * @author lixing
     * @version V1.0 2021/7/20 10:08 上午
     */
    void updateParagraphSupplement(String reportOutlineId, String paragraphId, List<AttrValueVO> supplement);
}
