package com.persagy.apm.report.indicator.utils;

import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;

/**
 * 指标工具类
 *
 * @author lixing
 * @version V1.0 2021/7/28 6:22 下午
 **/
public class IndicatorUtils {
    /**
     * 获取增幅  增量/基础值
     *
     * @param attrCode  属性编码
     * @param growth    增量
     * @param baseValue 基础值
     * @param valueType 数据类型
     * @return 增幅
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public static AttrValueVO getRange(String attrCode, Double growth, Double baseValue, AttrValueTypeEnum valueType) {
        Double range = null;
        if (growth != null && baseValue != null && baseValue != 0d) {
            range = DataUtils.doubleDivide(growth, baseValue);
        }
        String rangeStr = range == null ? null : String.valueOf(range);
        return new AttrValueVO(attrCode, rangeStr, valueType, DataConstants.PERCENT);
    }

    /**
     * 获取增量  value1-value2
     *
     * @param attrCode  属性编码
     * @param value1    用量1
     * @param value2    用量2
     * @param valueType 数据类型
     * @return 增量
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public static AttrValueVO getGrowth(String attrCode, Double value1, Double value2, AttrValueTypeEnum valueType) {
        Double growth = null;
        if (value1 != null && value2 != null) {
            growth = DataUtils.doubleSubtract(value1, value2);
        }

        String growthStr = growth == null ? null : String.valueOf(growth);
        return new AttrValueVO(attrCode, growthStr, valueType);
    }

    /**
     * 获取指标单方用量
     *
     * @param value 指标用量
     * @param area  面积
     * @return 指标单方用量
     * @author lixing
     * @version V1.0 2021/7/23 6:17 下午
     */
    public static Double getPerSqmValue(Double value, Double area) {
        Double perSqm = null;
        if (value != null && area != null && area != 0d) {
            perSqm = DataUtils.doubleDivide(value, area);
        }
        return perSqm;
    }

    /**
     * 获取段落属性对象的取值
     *
     * @param attrValueVO 段落属性对象
     * @return 属性对象的取值
     * @author lixing
     * @version V1.0 2021/8/13 11:28 上午
     */
    public static Double getAttrDoubleValue(AttrValueVO attrValueVO) {
        if (attrValueVO == null) {
            return null;
        }
        return DataUtils.parseDouble(attrValueVO.getValue());
    }

    /**
     * 获取段落属性对象的取值
     *
     * @param attrValueVO 段落属性对象
     * @return 属性对象的取值
     * @author lixing
     * @version V1.0 2021/8/13 11:28 上午
     */
    public static String getAttrStrValue(AttrValueVO attrValueVO) {
        if (attrValueVO == null) {
            return null;
        }
        return attrValueVO.getValue();
    }

    /**
     * 获取段落属性对象的单位
     *
     * @param attrValueVO 段落属性对象
     * @return 属性对象的单位
     * @author lixing
     * @version V1.0 2021/8/13 11:28 上午
     */
    public static String getAttrUnit(AttrValueVO attrValueVO) {
        if (attrValueVO == null) {
            return null;
        }
        return attrValueVO.getUnit();
    }
}
