package com.persagy.apm.report.userconfig.function.controller;

import com.persagy.apm.report.userconfig.function.service.IFunctionService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.report.userconfig.function.model.*;
import com.persagy.apm.report.userconfig.function.model.vo.*;
import com.persagy.apm.report.userconfig.function.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 功能点控制层
 *
 * @author lixing
 * @version V1.0 2021-07-23 17:55:17
 */
@Api(tags = "功能点")
@Validated
@RestController
@RequestMapping("functions")
@ApiIgnore
public class FunctionController {
    @Autowired
    private IFunctionService functionService;

    @ApiOperation(value = "创建功能点")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createFunction(
            @Valid @RequestBody AddFunctionDTO addFunctionDTO) {
        String id = functionService.createFunction(addFunctionDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "功能点详情")
    @PostMapping("/get")
    public CommonResult<FunctionItemVO> queryFunctionDetail(
            @Valid @RequestBody QueryFunctionDetailDTO queryFunctionDetailDTO) {
        Function function = functionService.queryFunctionDetail(queryFunctionDetailDTO.getId());
        FunctionItemVO functionItemVO = ConvertFunctionTool.INSTANCE.convert2ResponseItemDTO(function);
        return ResultHelper.single(functionItemVO);
    }


    @ApiOperation(value = "更新功能点")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateFunction(
            @Valid @RequestBody UpdateFunctionDTO updateFunctionDTO) {
        functionService.updateFunction(updateFunctionDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateFunctionDTO.getId()));
    }


    @ApiOperation(value = "删除功能点")
    @PostMapping("/delete")
    public CommonResult deleteFunction(
            @RequestBody @Valid DeleteFunctionDTO deleteFunctionDTO) {
        functionService.deleteFunction(deleteFunctionDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "功能点列表")
    @PostMapping("/query")
    public CommonResult<PageList<FunctionListItemVO>> queryFunction(
            @Valid @RequestBody QueryFunctionDTO queryFunctionDTO) {
        List<Function> functionList = functionService.queryFunctionList(queryFunctionDTO);
        List<FunctionListItemVO> resultList = ConvertFunctionTool.INSTANCE.convert2List(functionList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "功能点列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<FunctionListItemVO>> queryFunction(
    //        @Valid @RequestBody PageQueryFunctionDTO pageQueryFunctionDTO) {
    //    IPage<Function> pagedList = functionService.pageQueryFunction(pageQueryFunctionDTO);
    //    IPage<FunctionListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertFunctionTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，Function实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

