package com.persagy.apm.report.userconfig.function.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.report.userconfig.function.model.vo.*;
import com.persagy.apm.report.userconfig.function.model.dto.*;

import java.util.List;

/**
 * 功能点(Function) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-07-23 17:55:20 2021-07-23 17:55:20
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertFunctionTool {

    ConvertFunctionTool INSTANCE = Mappers.getMapper(ConvertFunctionTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param function do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-07-23 17:55:20
     */
    FunctionItemVO convert2ResponseItemDTO(Function function);

    /**
     * do转换为ResponseListItemVO
     *
     * @param function do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-07-23 17:55:20
     */
    FunctionListItemVO convert2ResponseListItemDTO(Function function);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param functionList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-07-23 17:55:20
     */
    List<FunctionListItemVO> convert2List(List<Function> functionList);

    /**
     * addDTO转换为do
     *
     * @param addFunctionDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-07-23 17:55:20
     */
    Function convertAddDto2Entity(AddFunctionDTO addFunctionDTO);

    /**
     * updateDTO转换为实体
     *
     * @param function          更新前的do对象
     * @param updateFunctionDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-07-23 17:55:20
     */
    Function convertUpdateDto2Entity(@MappingTarget Function function, UpdateFunctionDTO updateFunctionDTO);
}

