package com.persagy.apm.report.userconfig.paragraphcustom.controller;

import com.persagy.apm.report.userconfig.paragraphcustom.service.IParagraphCustomService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.report.userconfig.paragraphcustom.model.*;
import com.persagy.apm.report.userconfig.paragraphcustom.model.vo.*;
import com.persagy.apm.report.userconfig.paragraphcustom.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 段落定制控制层
 *
 * @author lixing
 * @version V1.0 2021-07-15 18:12:04
 */
@Api(tags = "段落定制")
@ApiIgnore
@Validated
@RestController
@RequestMapping("paragraphCustoms")
public class ParagraphCustomController {
    @Autowired
    private IParagraphCustomService paragraphCustomService;

    @ApiOperation(value = "创建段落定制")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createParagraphCustom(
            @Valid @RequestBody AddParagraphCustomDTO addParagraphCustomDTO) {
        String id = paragraphCustomService.createParagraphCustom(addParagraphCustomDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "段落定制详情")
    @PostMapping("/get")
    public CommonResult<ParagraphCustomItemVO> queryParagraphCustomDetail(
            @Valid @RequestBody QueryParagraphCustomDetailDTO queryParagraphCustomDetailDTO) {
        ParagraphCustom paragraphCustom = paragraphCustomService.queryParagraphCustomDetail(queryParagraphCustomDetailDTO.getId());
        ParagraphCustomItemVO paragraphCustomItemVO = ConvertParagraphCustomTool.INSTANCE.convert2ResponseItemDTO(paragraphCustom);
        return ResultHelper.single(paragraphCustomItemVO);
    }


    @ApiOperation(value = "更新段落定制")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateParagraphCustom(
            @Valid @RequestBody UpdateParagraphCustomDTO updateParagraphCustomDTO) {
        paragraphCustomService.updateParagraphCustom(updateParagraphCustomDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateParagraphCustomDTO.getId()));
    }


    @ApiOperation(value = "删除段落定制")
    @PostMapping("/delete")
    public CommonResult deleteParagraphCustom(
            @RequestBody @Valid DeleteParagraphCustomDTO deleteParagraphCustomDTO) {
        paragraphCustomService.deleteParagraphCustom(deleteParagraphCustomDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "段落定制列表")
    @PostMapping("/query")
    public CommonResult<PageList<ParagraphCustomListItemVO>> queryParagraphCustom(
            @Valid @RequestBody QueryParagraphCustomDTO queryParagraphCustomDTO) {
        List<ParagraphCustom> paragraphCustomList = paragraphCustomService.queryParagraphCustomList(queryParagraphCustomDTO);
        List<ParagraphCustomListItemVO> resultList = ConvertParagraphCustomTool.INSTANCE.convert2List(paragraphCustomList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "段落定制列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ParagraphCustomListItemVO>> queryParagraphCustom(
    //        @Valid @RequestBody PageQueryParagraphCustomDTO pageQueryParagraphCustomDTO) {
    //    IPage<ParagraphCustom> pagedList = paragraphCustomService.pageQueryParagraphCustom(pageQueryParagraphCustomDTO);
    //    IPage<ParagraphCustomListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertParagraphCustomTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，ParagraphCustom实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

