package com.persagy.apm.report.userconfig.reporttype.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.report.userconfig.paragraph.model.ConvertParagraphTool;
import com.persagy.apm.report.userconfig.paragraph.model.Paragraph;
import com.persagy.apm.report.userconfig.paragraph.model.vo.ParagraphItemVO;
import com.persagy.apm.report.userconfig.paragraph.service.IParagraphService;
import com.persagy.apm.report.userconfig.reporttype.constant.enums.BelongTypeEnum;
import com.persagy.apm.report.userconfig.reporttype.dao.ReportTypeMapper;
import com.persagy.apm.report.userconfig.reporttype.model.ConvertReportTypeTool;
import com.persagy.apm.report.userconfig.reporttype.model.ReportType;
import com.persagy.apm.report.userconfig.reporttype.model.dto.AddReportTypeDTO;
import com.persagy.apm.report.userconfig.reporttype.model.dto.PageQueryReportTypeDTO;
import com.persagy.apm.report.userconfig.reporttype.model.dto.QueryReportTypeDTO;
import com.persagy.apm.report.userconfig.reporttype.model.dto.UpdateReportTypeDTO;
import com.persagy.apm.report.userconfig.reporttype.model.vo.ResponseReportTypeListItemVO;
import com.persagy.apm.report.userconfig.reporttype.service.IReportTypeEnumService;
import com.persagy.apm.report.userconfig.reporttype.service.IReportTypeService;
import com.persagy.apm.report.dependencies.saasweb.model.vo.PartitionProjectVO;
import com.persagy.apm.report.dependencies.saasweb.model.vo.PartitionVO;
import com.persagy.apm.report.dependencies.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.report.dependencies.saasweb.service.ISaasWebService;
import com.persagy.apm.report.common.utils.ApmStrUtils;
import org.apache.commons.lang.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 报告类型(ReportType) service层
 *
 * @author lixing
 * @version V1.0 2021-05-19 10:39:15
 */
@Service
public class ReportTypeServiceImpl extends ServiceImpl<ReportTypeMapper, ReportType>
        implements IReportTypeService {
    @Autowired
    ISaasWebService saasWebService;
    @Autowired
    IReportTypeEnumService reportTypeEnumService;
    @Autowired
    IParagraphService paragraphService;

    /**
     * 创建报告类型
     *
     * @return 报告类型主键
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public String createReportType(AddReportTypeDTO addReportTypeDTO) {
        ReportType reportType = ConvertReportTypeTool.INSTANCE.convertAddDto2Entity(addReportTypeDTO);
        // 设置默认值
        setDefaultValue(reportType);
        save(reportType);
        return reportType.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportType 报告类型实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportType reportType) {
        reportType.setCreator(PoemsContext.getContext().getUserId());


    }

    /**
     * 报告类型详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public ReportType queryReportTypeDetail(String id) {
        ReportType reportType = getById(id);
        if (reportType == null) {
            throw new IllegalArgumentException("查看ReportType详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportType;
    }

    /**
     * 更新报告类型
     *
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public void updateReportType(UpdateReportTypeDTO updateReportTypeDTO) {
        ReportType reportType = getById(updateReportTypeDTO.getId());
        reportType = ConvertReportTypeTool.INSTANCE.convertUpdateDto2Entity(reportType, updateReportTypeDTO);
        reportType.setModifier(PoemsContext.getContext().getUserId());
        updateById(reportType);
    }

    /**
     * 校验报告类型是否可删除
     *
     * @param id 报告类型主键
     * @return 报告类型do类
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    public ReportType checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportType时发生异常，主键为空");
        }

        ReportType reportType = getById(id);

        if (reportType == null) {
            throw new IllegalArgumentException("删除ReportType时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportType;
    }

    /**
     * 删除报告类型
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public void deleteReportType(String id) {
        // 校验是否可删除
        ReportType reportType = checkDeletable(id);

        reportType.setValid(ValidEnum.FALSE.getType());
        updateById(reportType);
    }

    /**
     * 查询报告类型
     *
     * @return List<ReportType>
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public List<ReportType> queryReportTypeList(QueryReportTypeDTO queryReportTypeDTO) {
        QueryWrapper<ReportType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportType.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportType.PROP_CREATIONTIME);

        if (queryReportTypeDTO != null) {
            if (StringUtils.isNotEmpty(queryReportTypeDTO.getName())) {
                queryWrapper.like(ReportType.PROP_NAME, queryReportTypeDTO.getName());
            }

            if (queryReportTypeDTO.getBusinessType() != null) {
                queryWrapper.eq(ReportType.PROP_BUILDING_TYPE, queryReportTypeDTO.getBusinessType());
            }

            if (queryReportTypeDTO.getBelongType() != null) {
                queryWrapper.eq(ReportType.PROP_BELONG_TYPE, queryReportTypeDTO.getBelongType());
            }
        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告类型
     *
     * @return IPage<ReportType>
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    @Override
    public IPage<ReportType> pageQueryReportType(PageQueryReportTypeDTO pageQueryReportTypeDTO) {
        QueryWrapper<ReportType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportType.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportType> pageParam = new Page<>(pageQueryReportTypeDTO.getPage(), pageQueryReportTypeDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportTypeDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportType.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportTypeDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }


        if (StringUtils.isNotEmpty(pageQueryReportTypeDTO.getName())) {
            queryWrapper.like(ReportType.PROP_NAME, pageQueryReportTypeDTO.getName());
        }

        if (pageQueryReportTypeDTO.getBusinessType() != null) {
            queryWrapper.eq(ReportType.PROP_BUILDING_TYPE, pageQueryReportTypeDTO.getBusinessType());
        }

        if (pageQueryReportTypeDTO.getBelongType() != null) {
            queryWrapper.eq(ReportType.PROP_BELONG_TYPE, pageQueryReportTypeDTO.getBelongType());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public IPage<ResponseReportTypeListItemVO> queryAvailableReportTypes(String belongType) {
        // 返回结果初始化
        IPage<ResponseReportTypeListItemVO> result = new Page<>();
        List<ResponseReportTypeListItemVO> listItemVOList = Lists.newArrayList();
        result.setRecords(listItemVOList);
        result.setTotal(0L);

        // 获取对应所属类型所有的报告类型
        QueryReportTypeDTO queryReportTypeDTO = new QueryReportTypeDTO();
        queryReportTypeDTO.setBelongType(belongType);
        List<ReportType> allReportTypes = queryReportTypeList(queryReportTypeDTO);

        if (BelongTypeEnum.PROJECT.getType().equals(belongType)) {
            // 获取用户拥有的项目列表
            List<SimpleProjectVO> availableProjects = saasWebService.getAvailableProjects("");
            if (CollectionUtils.isEmpty(availableProjects) || CollectionUtils.isEmpty(allReportTypes)) {
                return result;
            }
            // 汇总这些项目的业态
            Set<String> functionTypes = availableProjects.stream().map(
                    SimpleProjectVO::getFunctionType).collect(Collectors.toSet());
            Map<String, List<String>> functionTypeProjectListMap = saasWebService.groupByFunctionType(availableProjects);

            // 遍历报告类型，根据业态是否有交集来过滤
            List<ReportType> filteredReportTypeList = allReportTypes.stream().filter(reportType -> {
                Set<String> reportTypeBuildingTypes = ApmStrUtils.str2Set(reportType.getBuildingTypes());
                List<String> relatedBuildingTypes = saasWebService.getRelatedBuildingTypes(
                        Lists.newArrayList(reportTypeBuildingTypes));
                relatedBuildingTypes.retainAll(functionTypes);
                return !CollectionUtils.isEmpty(relatedBuildingTypes);
            }).collect(Collectors.toList());
            // 结果封装
            listItemVOList = packageReportTypeListItemVOList(
                    filteredReportTypeList, functionTypeProjectListMap);
        } else if (BelongTypeEnum.AREA.getType().equals(belongType)) {
            for (ReportType reportType : allReportTypes) {
                List<String> availableBelongIds = saasWebService.getAvailableAreaIds(
                        ApmStrUtils.str2List(reportType.getBuildingTypes()));
                if (!CollectionUtils.isEmpty(availableBelongIds)) {
                    listItemVOList.add(packageReportTypeListItemVO(reportType, availableBelongIds));
                }
            }
        } else {
            throw new IllegalArgumentException("错误的报告所属类型");
        }
        result.setRecords(listItemVOList);
        result.setTotal(listItemVOList.size());

        return result;
    }

    /**
     * 包装reportTypeListItemVO列表
     *
     * @param reportTypes 报告类型对象列表
     * @return 包装后的reportTypeListItemVO列表
     * @author lixing
     * @version V1.0 2021/6/23 5:39 下午
     */
    private List<ResponseReportTypeListItemVO> packageReportTypeListItemVOList(
            List<ReportType> reportTypes, Map<String, List<String>> functionTypeBelongMap) {
        if (CollectionUtils.isEmpty(reportTypes)) {
            return Lists.newArrayList();
        }
        List<ResponseReportTypeListItemVO> results = new ArrayList<>();
        for (ReportType reportType : reportTypes) {
            ResponseReportTypeListItemVO responseReportTypeListItemVO = ConvertReportTypeTool.INSTANCE.
                    convert2ResponseListItemDTO(reportType);
            // 统计报告类型下可选的所属对象
            String buildingTypes = reportType.getBuildingTypes();
            List<String> buildingTypeList = ApmStrUtils.str2List(buildingTypes);
            List<String> relatedBuildingTypes = saasWebService.getRelatedBuildingTypes(buildingTypeList);
            List<String> availableBelongIds = new ArrayList<>();
            for (String buildingType : relatedBuildingTypes) {
                if (!functionTypeBelongMap.containsKey(buildingType)) {
                    continue;
                }
                availableBelongIds.addAll(functionTypeBelongMap.get(buildingType));
            }
            // 获取用户在报告类型下的可选所属
            responseReportTypeListItemVO.setAvailableBelongIds(availableBelongIds);
            results.add(responseReportTypeListItemVO);
        }
        return results;
    }

    /**
     * 包装reportTypeListItemVO列表
     *
     * @param reportType 报告类型对象
     * @param availableBelongIds 报告类型下可选所属对象
     * @return 包装后的reportTypeListItemVO
     * @author lixing
     * @version V1.0 2021/6/23 5:39 下午
     */
    private ResponseReportTypeListItemVO packageReportTypeListItemVO(
            ReportType reportType, List<String> availableBelongIds) {
        if (reportType == null) {
            return null;
        }
        ResponseReportTypeListItemVO responseReportTypeListItemVO = ConvertReportTypeTool.INSTANCE.
                convert2ResponseListItemDTO(reportType);
        // 获取用户在报告类型下的可选所属
        responseReportTypeListItemVO.setAvailableBelongIds(availableBelongIds);
        return responseReportTypeListItemVO;
    }

    @Override
    public List<String> getReportTypeBuildingTypes(String reportTypeId) {
        ReportType reportType = queryReportTypeDetail(reportTypeId);
        String buildingTypes = reportType.getBuildingTypes();
        String splitFlag = ",";
        if (StringUtils.isNotBlank(buildingTypes)) {
            return Lists.newArrayList(buildingTypes.split(splitFlag));
        }
        return null;
    }

    @Override
    public List<PartitionProjectVO> getAvailableProjectTree(String reportTypeId) {
        // 获取报告类型关联的业态
        List<String> buildingTypes = getReportTypeBuildingTypes(reportTypeId);
        return saasWebService.getAvailableProjectTree(buildingTypes);
    }

    @Override
    public List<SimpleProjectVO> queryAvailableProjectList(String reportTypeId) {
        // 获取报告类型关联的业态
        List<String> buildingTypes = getReportTypeBuildingTypes(reportTypeId);
        return saasWebService.getAvailableProjects(buildingTypes);
    }

    @Override
    public List<PartitionVO> getAvailableAreaTree(String reportTypeId) {
        // 获取报告类型关联的业态
        List<String> buildingTypes = getReportTypeBuildingTypes(reportTypeId);
        return saasWebService.getAvailableAreaTree(buildingTypes);
    }

    @Override
    public List<PartitionVO> getSonAreas(String reportTypeId, String areaId) {
        // 获取报告类型关联的业态
        List<String> buildingTypes = getReportTypeBuildingTypes(reportTypeId);
        return saasWebService.getSonAreas(buildingTypes, areaId);
    }

    @Override
    public List<ParagraphItemVO> queryParagraphsByReportType(String reportTypeId) {
        ReportType reportType = queryReportTypeDetail(reportTypeId);
        String paragraphs = reportType.getParagraphs();

        if (StringUtils.isBlank(paragraphs)) {
            return null;
        }
        String splitFlag = ",";
        String[] paragraphArray = paragraphs.split(splitFlag);
        List<ParagraphItemVO> results = new ArrayList<>();
        for (String paragraphId : paragraphArray) {
            Paragraph paragraph = paragraphService.queryParagraphDetail(paragraphId);
            results.add(ConvertParagraphTool.INSTANCE.convert2ResponseItemDTO(paragraph));
        }
        return results;
    }
}
