package com.persagy.apm.report.userconfig.statisticsitems.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.report.userconfig.statisticsitems.dao.StatisticItemMapper;
import com.persagy.apm.report.userconfig.statisticsitems.model.ConvertStatisticItemTool;
import com.persagy.apm.report.userconfig.statisticsitems.model.StatisticItem;
import com.persagy.apm.report.userconfig.statisticsitems.model.dto.AddStatisticItemDTO;
import com.persagy.apm.report.userconfig.statisticsitems.model.dto.PageQueryStatisticItemDTO;
import com.persagy.apm.report.userconfig.statisticsitems.model.dto.QueryStatisticItemDTO;
import com.persagy.apm.report.userconfig.statisticsitems.model.dto.UpdateStatisticItemDTO;
import com.persagy.apm.report.userconfig.statisticsitems.service.IStatisticItemService;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 统计条目(StatisticItem) service层
 *
 * @author lixing
 * @version V1.0 2021-07-27 16:11:57
 */
@Service
public class StatisticItemServiceImpl extends ServiceImpl<StatisticItemMapper, StatisticItem>
        implements IStatisticItemService {

    /**
     * 创建统计条目
     *
     * @return 统计条目主键
     * @author lixing
     * @version V1.0 2021-07-27 16:11:57
     */
    @Override
    public String createStatisticItem(AddStatisticItemDTO addStatisticItemDTO) {
        StatisticItem statisticItem = ConvertStatisticItemTool.INSTANCE.convertAddDto2Entity(addStatisticItemDTO);
        // 设置默认值
        setDefaultValue(statisticItem);
        save(statisticItem);
        return statisticItem.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param statisticItem 统计条目实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(StatisticItem statisticItem) {
        statisticItem.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 统计条目详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-07-27 16:11:57
     */
    @Override
    public StatisticItem queryStatisticItemDetail(String id) {
        StatisticItem statisticItem = getById(id);
        if (statisticItem == null) {
            throw new IllegalArgumentException("查看StatisticItem详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return statisticItem;
    }

    /**
     * 更新统计条目
     *
     * @author lixing
     * @version V1.0 2021-07-27 16:11:57
     */
    @Override
    public void updateStatisticItem(UpdateStatisticItemDTO updateStatisticItemDTO) {
        StatisticItem statisticItem = getById(updateStatisticItemDTO.getId());
        statisticItem = ConvertStatisticItemTool.INSTANCE.convertUpdateDto2Entity(statisticItem, updateStatisticItemDTO);
        statisticItem.setModifier(AppContext.getContext().getAccountId());
        updateById(statisticItem);
    }

    /**
     * 校验统计条目是否可删除
     *
     * @param id 统计条目主键
     * @return 统计条目do类
     * @author lixing
     * @version V1.0 2021-07-27 16:11:57
     */
    public StatisticItem checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除StatisticItem时发生异常，主键为空");
        }

        StatisticItem statisticItem = getById(id);

        if (statisticItem == null) {
            throw new IllegalArgumentException("删除StatisticItem时发生异常，找不到要删除的数据，id:" + id);
        }

        return statisticItem;
    }

    /**
     * 删除统计条目
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-27 16:11:57
     */
    @Override
    public void deleteStatisticItem(String id) {
        // 校验是否可删除
        StatisticItem statisticItem = checkDeletable(id);

        statisticItem.setValid(ValidEnum.FALSE.getType());
        updateById(statisticItem);
    }

    /**
     * 查询统计条目
     *
     * @return List<StatisticItem>
     * @author lixing
     * @version V1.0 2021-07-27 16:11:57
     */
    @Override
    public List<StatisticItem> queryStatisticItemList(QueryStatisticItemDTO queryStatisticItemDTO) {
        QueryWrapper<StatisticItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StatisticItem.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, StatisticItem.PROP_CREATIONTIME);

        if (queryStatisticItemDTO != null) {

            if (StringUtils.isNotEmpty(queryStatisticItemDTO.getCode())) {
                queryWrapper.eq(StatisticItem.PROP_CODE, queryStatisticItemDTO.getCode());
            }

            if (StringUtils.isNotEmpty(queryStatisticItemDTO.getStandardValue())) {
                queryWrapper.eq(StatisticItem.PROP_STANDARD_VALUE, queryStatisticItemDTO.getStandardValue());
            }

            if (StringUtils.isNotEmpty(queryStatisticItemDTO.getQualifyFormula())) {
                queryWrapper.eq(StatisticItem.PROP_QUALIFY_FORMULA, queryStatisticItemDTO.getQualifyFormula());
            }

            if (StringUtils.isNotEmpty(queryStatisticItemDTO.getStandardFlag())) {
                queryWrapper.eq(StatisticItem.PROP_STANDARD_FLAG, queryStatisticItemDTO.getStandardFlag());
            }
        }

        return list(queryWrapper);
    }

    /**
     * 分页查询统计条目
     *
     * @return IPage<StatisticItem>
     * @author lixing
     * @version V1.0 2021-07-27 16:11:57
     */
    @Override
    public IPage<StatisticItem> pageQueryStatisticItem(PageQueryStatisticItemDTO pageQueryStatisticItemDTO) {
        QueryWrapper<StatisticItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StatisticItem.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<StatisticItem> pageParam = new Page<>(pageQueryStatisticItemDTO.getPage(), pageQueryStatisticItemDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryStatisticItemDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, StatisticItem.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryStatisticItemDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        if (StringUtils.isNotEmpty(pageQueryStatisticItemDTO.getCode())) {
            queryWrapper.eq(StatisticItem.PROP_CODE, pageQueryStatisticItemDTO.getCode());
        }

        if (StringUtils.isNotEmpty(pageQueryStatisticItemDTO.getStandardValue())) {
            queryWrapper.eq(StatisticItem.PROP_STANDARD_VALUE, pageQueryStatisticItemDTO.getStandardValue());
        }

        if (StringUtils.isNotEmpty(pageQueryStatisticItemDTO.getQualifyFormula())) {
            queryWrapper.eq(StatisticItem.PROP_QUALIFY_FORMULA, pageQueryStatisticItemDTO.getQualifyFormula());
        }

        if (StringUtils.isNotEmpty(pageQueryStatisticItemDTO.getStandardFlag())) {
            queryWrapper.eq(StatisticItem.PROP_STANDARD_FLAG, pageQueryStatisticItemDTO.getStandardFlag());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public StatisticItem getItemByCode(String code) {
        if (StringUtils.isBlank(code)) {
            return null;
        }
        QueryStatisticItemDTO queryStatisticItemDTO = new QueryStatisticItemDTO();
        queryStatisticItemDTO.setCode(code);
        List<StatisticItem> statisticItems = queryStatisticItemList(queryStatisticItemDTO);
        if (CollectionUtils.isEmpty(statisticItems)) {
            return null;
        }
        return statisticItems.get(0);
    }
}
