package com.persagy.apm.report.dependencies.iotdataanalysis.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.common.utils.HttpUtils;
import com.persagy.apm.report.dependencies.iotdataanalysis.service.IDataAnalysisWebService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class IDataAnalysisWebServiceImpl implements IDataAnalysisWebService {

    @Value("${iot.data.analysis.url}")
    private String iotDataAnalysisUrl;
    @Value("${group.code}")
    private String groupCode;

    @Override
    public Double getTargetValue(String projectId, Date date, String targetCode) {
        try {
            JSONObject paramObject = new JSONObject();
            paramObject.put("userId", DataConstants.USER_ID);
//                        paramObject.put("groupCode", "BL");
//                        projectId = "Pj1101080259";
            paramObject.put("groupCode", groupCode);
            paramObject.put("projectIdList", Lists.newArrayList(projectId));
            // 加入分页条件，接口要求分页条件必传
            paramObject.put("pageIndex", 1);
            paramObject.put("pageSize", 10);
            Date startTime = DateUtils.getFirstDayOfMonth(date);
            Date endTime = DateUtils.getMonthOff(startTime, 1);
            paramObject.put("alarmStartTime", startTime);
            paramObject.put("alarmEndTime", endTime);
            String url = iotDataAnalysisUrl + "/alarm/show/view/queryList";
            String response = HttpUtils.postJson(url, paramObject.toString());
            JSONObject responseObject = JSONObject.parseObject(response);
            if (DataConstants.SUCCESS.equals(responseObject.getString("respMsg"))) {
                List<Map> content = (List<Map>) responseObject.get("content");
                if (CollectionUtils.isNotEmpty(content)) {
                    Map data = content.get(0);
                    if (null != data) {
                        return DataUtils.parseDouble(data.get(targetCode));
                    }
                }
            }
        } catch (Exception e) {
            throw new RuntimeException("获取数据质量服务变损数据异常", e);
        }
        return null;
    }


}
